//
//  BlockedCallList.cpp
//
//  Allows query access to a blocked call list, which auto-updates when
//  the file containing the list of calls is changed.
//
//  Matt Roberts, KK5JY, 2021-05-04.
//

#include "blockedcalls.h"
#include <iostream>
#include <QFile>
#include <QFileInfo>
#include <QTextStream>

#define EMIT_CERR_DETAILS

BlockedCallList::BlockedCallList(QString path)
	: _theFile(path) {
	update();
}

bool BlockedCallList::isBlocked(QString call) {
	if (!update())
		return false;
	std::list<QString>::const_iterator i
		= std::find(_blocked.begin(), _blocked.end(), call);
	bool result = (i != _blocked.end());
#ifdef EMIT_CERR_DETAILS
	std::cerr
		<< "INFO: isBlocked(" << call.toStdString() << ") -> "
		<< (result ? "BLOCKED" : "OK") << std::endl;
#endif // EMIT_CERR_DETAILS
	return result;
}

bool BlockedCallList::update() {
	try {
		QFileInfo fi(_theFile);

		if (! fi.exists())
			return true;

		if (fi.lastModified() == _lastMod)
			return true;

#ifdef EMIT_CERR_DETAILS
		std::cerr << "INFO: Reading new/updated file: " << _theFile.toStdString() << std::endl;
#endif // EMIT_CERR_DETAILS
		QFile f(_theFile);
		bool fail = false;
		if (f.open(QIODevice::ReadOnly)) {
			_lastMod = fi.lastModified();
			try {
				_blocked.clear();

				// read all the lines
				QTextStream in(&f);
				while (!in.atEnd()) {
					// read one line
					QString line = in.readLine();

					// trim off trailing comments
					int i = line.indexOf(QChar('#'));
					if (i >= 0) {
						line = line.left(i);
					}

					// remove leading and trailing whitespace
					line = line.trimmed();

					// if there is anything left, store that as a call
					if (line.length() != 0) {
						if (line[0] != '#')
							_blocked.push_back(line);
					}
				}
#ifdef EMIT_CERR_DETAILS
				std::cerr << "INFO: update() processed " << _blocked.size() << " calls."<< std::endl;
#endif // EMIT_CERR_DETAILS
			} catch (const std::exception &ex) {
				std::cerr << "BlockedCallList::update() failed: " << ex.what() << std::endl;
				fail = true;
			}

			try {
				f.close();
			} catch (const std::exception &ex) {
				fail = true;
			}
		}

		return ! fail;
	} catch (const std::exception &ex) {
		std::cerr << "BlockedCallList::update() failed: " << ex.what() << std::endl;
		return false;
	}
}
